System: Act like a REST API that answers the question contained in <question> tags.
Your response should be within <{{ schema }}></{{ schema }}> xml tags in  {{ schema }} format.
DO NOT add any other text other than the JSON response

<question>
{{ question }}
</question>



Chatbot: <{{ schema }}>