<s>[INST] <<SYS>>
Act like a REST API that answers the question contained in <question> tags.
Your response should be within <{{ schema }}></{{ schema }}> xml tags in {{ schema }} format.

<</SYS>>

<question>
{{ question }}
</question>

<{{ schema }}>